from tuya_iot import TuyaOpenAPI, AuthType, TuyaDeviceManager, TuyaOpenMQ

from .device import GosundDevice
from .exceptions import assert_response_success

class Gosund(object):

    def __init__(self, username, password, access_id, access_key, country=1,
            endpoint='https://openapi.tuyaus.com'):
        self.api = TuyaOpenAPI(endpoint, access_id, access_key,
                auth_type=AuthType.CUSTOM)
        resp = self.api.connect(username, password, country)
        assert_response_success('connect to api', resp)
        self.manager = TuyaDeviceManager(self.api, TuyaOpenMQ(self.api))

    def get_device(self, device_id):
        resp = self.manager.get_device_functions(device_id)
        assert_response_success('get device', resp)
        return GosundDevice.from_response(resp, device_id, self.manager)



from .exceptions import assert_response_success

class GosundDevice(object):

    @staticmethod
    def from_response(resp, device_id, manager):
        category = resp['result']['category']
        if category == 'cz':
            cls = GosundSwitchDevice
        elif category == 'dj':
            cls = GosundLightbulbDevice
        else:
            cls = GosundDevice
        return cls(device_id, manager)

    def __init__(self, device_id, manager):
        self.device_id = device_id
        self.manager = manager

    def get_status(self):
        resp = self.manager.get_device_status(self.device_id)
        assert_response_success('get device status', resp)
        return resp.get('result', [])

    def send_commands(self, commands):
        resp = self.manager.send_commands(self.device_id, commands)
        codes = [f'{cmd["code"]}:{cmd["value"]}' for cmd in commands]
        assert_response_success(f'send {codes} command to device', resp)
        return resp

    def __str__(self):
        return f'<{self.__class__.__name__} device_id={self.device_id}>'
    __repr__ = __str__

class DeviceOnOffMixin(object):

    def turn_on(self):
        commands = [{'code': self.on_off_code, 'value': True}]
        return self.send_commands(commands)

    def turn_off(self):
        commands = [{'code': self.on_off_code, 'value': False}]
        return self.send_commands(commands)

    def is_on(self):
        for status in self.get_status():
            if status.get('code') == self.on_off_code:
                return status.get('value', False)
        return False

    def switch(self):
        value = not self.is_on()
        commands = [{'code': self.on_off_code, 'value': value}]
        return self.send_commands(commands)

class GosundSwitchDevice(GosundDevice, DeviceOnOffMixin):

    on_off_code = 'switch_1'

class GosundLightbulbDevice(GosundDevice, DeviceOnOffMixin):

    on_off_code = 'switch_led'



import os

if os.environ.get('GOSUND_DEBUG') in ('True', 'true', '1'):
    import logging, tuya_iot
    tuya_iot.TUYA_LOGGER.setLevel(logging.DEBUG)

from .device import GosundDevice, GosundSwitchDevice, GosundLightbulbDevice
from .exceptions import GosundException
from .gosund import Gosund
from .version import version

__all__ = [
        'GosundDevice',
        'GosundSwitchDevice',
        'GosundLightbulbDevice',
        'GosundException',
        'Gosund',
]

__version__ = version



class GosundException(Exception):
    pass

def assert_response_success(action, response):
    if not response.get('success'):
        err = response.get('msg', 'unknown')
        raise GosundException(f'unable to {action}: {err}')



import codecs
import os.path

from setuptools import find_packages, setup

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('version'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

setup(
    name="gosundpy",
    version=get_version("gosundpy/version.py"),
    url="https://github.com/purple4reina/gosundpy",
    author="Rey Abolofia",
    author_email="purple4reina@gmail.com",
    keywords="gosund,smartlife,tuya,iot,api,sdk,python",
    description="Python API for controling Gosund smart devices",
    license="MIT",
    project_urls={
        "Source": "https://github.com/purple4reina/gosundpy",
        "Bug Tracker": "https://github.com/purple4reina/gosundpy/issues",
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    install_requires=["tuya-iot-py-sdk==0.6.6"],
    packages=find_packages(),
    python_requires=">=3.7, <4",
)
